#!/bin/bash

# A simple diagnostic script reporting outages in SPI communication
# unipi.technology @ 2021


# ==== Variables section ====
REPORT_TO_SLACK='y'
REPORT_TO_DB='y'
serial="" # Optional interconnection with DUT via an external RS485 line
#EXAMPLE serial="/dev/ttyNS0"
#============================

counter=0
counter_reset=60

#Unipi notifikator channel
#SLACK_URL="https://hooks.slack.com/services/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"

# Dead beef channel
SLACK_URL="https://hooks.slack.com/services/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"
log_file="/opt/comm_outages.log"
version_str="0.4" # Placeholder to be replaced in CI script by the right value

sleep 2

function remote_report_msg {
    #Remote log server
    if [[ -n $REPORT_TO_DB ]]; then
      logger -P 514 -n 81.19.0.22 -d -t BRAIN -p 2 --rfc5424=notq "${1}"
    fi

    #Slack API
    if [[ -n $REPORT_TO_SLACK ]]; then
      machine_name=$(hostname)
      curl -X POST -H 'Content-type: application/json' --data "{\"text\": \" ${machine_name}(v.${version_str}): ${1}\n\"}" "$SLACK_URL"
    fi
}

if [ $(< /proc/uptime read a b && echo $((${a%.*}))) -le 60 ]; then
        echo "$(date "+%Y-%m-%d %H:%M:%S")  BOOT (fw-diag-tool v.${version_str})" >> /opt/vypadky.log
        remote_report_msg "BOOT (fw-diag-tool v.${version_str})"
fi

# Verify serial line
if [[ ! -c "${serial}" ]]; then
    serial=""
fi

while true; do
    if ! mbpoll 127.0.0.1 -a 1 -0 -1 -r 1000 -q >/dev/null 2>/dev/null; then
        echo "$(date "+%Y-%m-%d %H:%M:%S")" >> $log_file
        # Serial test is present
        if [ -n "${serial}" ]; then
            mb=$(mbpoll /dev/ttyNS0 -a 15 -P none -b 9600 -0 -r 1033 -c 13 -1 -q 2>&1)
            echo "$mb" >> $log_file
        fi

        echo "Counter: $counter" >> $log_file
        if [ "$counter" == "0" ]; then
            if [ -n "${serial}" ]; then
                remote_report_msg "${mb}"
            else
                remote_report_msg "SPI OUTAGE"
            fi
            counter=$counter_reset
        fi
        counter=$((counter-1))

    else
        if mbpoll 127.0.0.1 -a 1 -01 -r 1000 -q -t0 2>/dev/null | grep -q -e '\s1$'; then
            # reset watchdog bit
            mbpoll 127.0.0.1 -a 1 -01 -r 1000 -q -t0 1 2>/dev/null >/dev/null
            echo "$(date) Watchdog reboot" >> $log_file
            remote_report_msg "Watchdog RESET!"
        fi

        # reset counter, so next error will send message immediatelly
        counter=0
    fi
        sleep 1
done
